@ ------------------------------------------------------------------------- @
@ JOHA1.prc                                                                 @
@  Procedure to Estimate 1 cointegrating vector using Johansen's MLE, and   @
@  optionally to construct an LR statistic concerning its value.            @
@  Output includes Beta, its covariance matrix and the LR test statistic.   @
@ ------------------------------------------------------------------------- @
@ ------------------------------------------------------------------------- @
@ Regress X(t-k) & dX(t) on lagged differences => residuals R(kt) & R(0t)   @
@ ------------------------------------------------------------------------- @
PROC (4)=jovar(x,nfirst,nlast,nlag,optn);
local *;

dx = zeros(1,cols(x))|x[2:nlast,.]-x[1:nlast-1,.];
rtx = zeros(nlast-nfirst+1,1 );

n=1; do while n <= nlag;
   rtx = rtx~dx[nfirst-n:nlast-n,.];
n=n+1; endo;
@ Detrending Options @
    if (optn .eq 2); rtx=rtx~ones(rows(rtx),1);
elseif (optn .eq 3); rtx=rtx~ones(rows(rtx),1)~seqa(1,1,rows(rtx));
endif;
rtx = rtx[.,2:cols(rtx)];               @ Eliminate Column of zeros @

@ Form Residual series @

ri=invpd(rtx'rtx);
x0=dx[nfirst:nlast,.];
xk= x[nfirst-nlag-1:nlast-nlag-1,.];
r0 = x0 - rtx*ri*(rtx'x0);
rk = xk - rtx*ri*(rtx'xk);

@ Form Moment Matrices @
ndf1=rows(r0)-cols(rtx);
s00 = (r0'r0)/ndf1;
skk = (rk'rk)/ndf1;
sk0 = (rk'r0)/ndf1;

RETP(s00,sk0,skk,ndf1); ENDP;

@ ------------------------------------------------------------------------- @
@ Procedure CANON calculates canonical vectors and squared correlations,    @
@ based on the straightforward eigen decomposition.                         @
@ The procedure returns:                                                    @
@   (1) a vector of squared canonical correlations, and                     @
@   (2) a matrix of canonical coefficients (cointegrating vectors)          @
@ Simplified version by Ken Kuttner 1-16-89.                                @
@ ------------------------------------------------------------------------- @
PROC (2)=canon(s00,sk0,skk);
local *;

pk   = cols(skk);                       @ Useful when imposing constraints @
flip = design(seqa(pk,-1,pk));          @ Useful to flip the column order  @

           ww = chol( invpd(skk) );                     @ Make this a       @
{lmbd,gtilde} = eigrs2( ww*sk0*invpd(s00)*sk0'*ww' );   @ symmetric eigen   @
         beta = ww'gtilde;                              @ problem...        @

lmbd = flip*lmbd;                 @ Rearrange eigenvectors & eigenvalues    @
beta = beta*flip;                 @ to run from largest to smallest... and  @
beta = beta ./ beta[1,.];         @ Normalize beta s.t. 1st element = 1     @

retp(lmbd, beta);
endp;

@ ------------------------------------------------------------------------- @
@ Test beta = H*phi                                                         @
@ ------------------------------------------------------------------------- @
PROC(3)=hrest(lmbd1,lmbd2,ndf,ah,r);
local tst,df,pv;

tst = ndf*sumc( ln( (1- lmbd2[1:r,1])./(1- lmbd1[1:r,1]) ));
df  = r*(rows(lmbd1)-cols(ah));
pv  = 100*(cdfchic(tst,df));

RETP(tst,pv,df); ENDP;

@ -------------------------------------------------------------------------@
@ NLLS @
@ -------------------------------------------------------------------------@
proc (2)=nlls1(s00,sk0,skk,beta2,ndf1);
local *;
rki=invpd(skk);
b=rki*(sk0*beta2);
betj=b[2:rows(b),1]./b[1,1];
s2=(beta2's00*beta2-beta2'sk0'*b)/(ndf1-cols(s00));
r=(-betj~eye(rows(betj)))./(-b[1,1]);
varb=s2*r*rki*r';
retp(betj,varb);
endp;

@ ------------------------------------------------------------------------- @
@ Main program.                                                             @
@ ------------------------------------------------------------------------- @
PROC (3)=joha1(y,x,nfirst,nlast,nlag,optn,ictst,ah,r);
@
 Input
      y = y data              Tx1
      x = x data (levels of ) Txk
      nfirst = first obs of y and x to use in regression
      nlast = last obs of y and x to use in regression
      nlag = number of lags of delta x to use in regression
        (notice that lags include values of x before nfirst)
      optn = detrending option
                 1 == nothing
                 2 == include constant
                 3 == include constant and time trend
      ictst  = 1 to test Cointegrating vector, 0 otherwise
      ah  = Null of cointegrating space
      r  = number of cointegrating vectors to be tested

 Output:
      Beta = cointegrating vector
      VARB = covaraince matrix
      lrtst = LR statistic of testing null space of cointegrating vector.

@
local *;
lrtst=0;
           x = y~x;
{s00,sk0,skk,ndf1} = jovar(x,nfirst,nlast,nlag,optn);
{lmbd1, beta} = canon(s00,sk0,skk);
@ Carry Out LR test @

if ictst > 0;
   ask0=ah'sk0;askka=ah'skk*ah;
   {lmbd2,junk} = canon(s00,ask0,askka);
   ndf=nlast-nfirst+1-cols(s00);
   {lrtst,pv2,df} = hrest(lmbd1,lmbd2,ndf,ah,r);
endif;


@ Estimate Covariance Matrix @
{junk,beta2}= canon(skk,sk0',s00);
@ Convert covariance matrices to Sum of Squares @
skk=skk*ndf1;
s00=s00*ndf1;
sk0=sk0*ndf1;
{betaj,varb}=nlls1(s00,sk0,skk,beta2[.,1],ndf1);
RETP(betaj,varb,lrtst);

ENDP;

/*
@ debug: @
   ictst=1;optn=2;ah=1|-1|0;r=1;
   nf=  4 ;nl= 88 ;nlag=2 ;
    {betaj,varb,lrtst}=joha1(y,x,nf,nl,nlag,optn,ictst,ah,r);
    betaj;varb;lrtst;
*/

